@@ -18,8 +18,9 @@ try:
 except ImportError:
     import urllib2
 import json
+from . import util
 
-__version__ = '0.23'
+__version__ = '0.28'
 
 
 class __UnknownValue(object):
@@ -73,6 +74,8 @@ class System(object):
        :param str id: Unique identifier for this system in the mmCIF file.
     """
 
+    structure_determination_methodology = "integrative"
+
     def __init__(self, title=None, id='model'):
         self.id = id
         self.title = title
@@ -86,7 +89,7 @@ class System(object):
 
         #: List of all authors of this system, as a list of strings (last name
         #: followed by initials, e.g. "Smith AJ"). When writing out a file,
-        #: if this is list is empty, the set of all citation authors (see
+        #: if this list is empty, the set of all citation authors (see
         #: :attr:`Citation.authors`) is used instead.
         self.authors = []
 
@@ -211,6 +214,26 @@ class System(object):
         #: See :class:`~ihm.flr.FLRData`.
         self.flr_data = []
 
+    def _make_complete_assembly(self):
+        """Fill in the complete assembly with all asym units"""
+        # Clear out any existing components
+        self.complete_assembly[:] = []
+
+        # Include all asym units
+        for asym in self.asym_units:
+            self.complete_assembly.append(asym)
+
+    def _all_models(self):
+        """Iterate over all Models in the system"""
+        # todo: raise an error if a model is present in multiple groups
+        for group in self._all_model_groups():
+            seen_models = {}
+            for model in group:
+                if model in seen_models:
+                    continue
+                seen_models[model] = None
+                yield group, model
+
     def update_locations_in_repositories(self, repos):
         """Update all :class:`Location` objects in the system that lie within
            a checked-out :class:`Repository` to point to that repository.
@@ -283,17 +306,6 @@ class System(object):
                         yield edge.group_begin
                         yield edge.group_end
 
-    def _all_models(self):
-        """Iterate over all Models in the system"""
-        # todo: raise an error if a model is present in multiple groups
-        for group in self._all_model_groups():
-            seen_models = {}
-            for model in group:
-                if model in seen_models:
-                    continue
-                seen_models[model] = None
-                yield group, model
-
     def _all_representations(self):
         """Iterate over all Representations in the system.
            This includes all Representations referenced from other objects,
@@ -529,14 +541,26 @@ class System(object):
                 for comp in f._all_entities_or_asyms()),
             (d.asym_unit for d in self._all_densities())))
 
-    def _make_complete_assembly(self):
-        """Fill in the complete assembly with all asym units"""
-        # Clear out any existing components
-        self.complete_assembly[:] = []
+    def _before_write(self):
+        """Do any setup necessary before writing out to a file"""
+        # Here, we initialize all RestraintGroups by removing any assigned ID
+        for g in self.restraint_groups:
+            util._remove_id(g)
+        # Fill in complete assembly
+        self._make_complete_assembly()
 
-        # Include all asym units
-        for asym in self.asym_units:
-            self.complete_assembly.append(asym)
+    def _check_after_write(self):
+        """Make sure everything was successfully written"""
+        # Here, we check that all RestraintGroups were successfully dumped"""
+        for g in self.restraint_groups:
+            if len(g) > 0 and not hasattr(g, '_id'):
+                raise TypeError(
+                    "RestraintGroup(%s) contains an unsupported combination "
+                    "of Restraints. Due to limitations of the underlying "
+                    "dictionary, all objects in a RestraintGroup must be of "
+                    "the same type, and only certain types (currently only "
+                    "DerivedDistanceRestraint or PredictedContactRestraint) "
+                    "can be grouped." % g)
 
 
 class Software(object):
@@ -607,20 +631,27 @@ class Citation(object):
        :param str pmid: The PubMed ID.
        :param str title: Full title of the publication.
        :param str journal: Abbreviated journal name.
-       :param int volume: Journal volume number.
+       :param volume: Journal volume as int for a plain number or str for
+                      journals adding a label to the number (e.g. "46(W1)" for
+                      a web server issue).
        :param page_range: The page (int) or page range (as a 2-element
-              int tuple).
+              int tuple). Using str also works for labelled page numbers.
        :param int year: Year of publication.
        :param authors: All authors in order, as a list of strings (last name
               followed by initials, e.g. "Smith AJ").
        :param str doi: Digital Object Identifier of the publication.
+       :param bool is_primary: Denotes the most pertinent publication for the
+              modeling itself (as opposed to a method or piece of software used
+              in the protocol). Only one such publication is allowed, and it
+              is assigned the ID "primary" in the mmCIF file.
     """
     def __init__(self, pmid, title, journal, volume, page_range, year, authors,
-                 doi):
+                 doi, is_primary=False):
         self.title, self.journal, self.volume = title, journal, volume
         self.page_range, self.year = page_range, year
         self.pmid, self.doi = pmid, doi
         self.authors = authors if authors is not None else []
+        self.is_primary = is_primary
 
     @classmethod
     def from_pubmed_id(cls, pubmed_id):
@@ -737,8 +768,9 @@ class ChemComp(object):
             emass = self._element_mass.get(m.group(1), None)
             if emass:
                 weight += emass * (int(m.group(2)) if m.group(2) else 1)
-            else:
+            elif m.group(1) != 'X':
                 # If element is unknown, weight is unknown too
+                # Element 'X' is used for GLX/ASX and has zero weight
                 return None
         return weight
 
@@ -788,19 +820,22 @@ class RNAChemComp(ChemComp):
 
 
 class NonPolymerChemComp(ChemComp):
-    """A non-polymer chemical component, such as a ligand
-       (for crystal waters, use :class:`WaterChemComp`).
+    """A non-polymer chemical component, such as a ligand or a non-standard
+       residue (for crystal waters, use :class:`WaterChemComp`).
 
        :param str id: A globally unique identifier for this component.
+       :param str code_canonical: Canonical one-letter identifier. This is
+              used for non-standard residues and should be the one-letter code
+              of the closest standard residue (or by default, 'X').
        :param str name: A longer human-readable name for the component.
        :param str formula: The chemical formula. See :class:`ChemComp` for
               more details.
     """
     type = "non-polymer"
 
-    def __init__(self, id, name=None, formula=None):
-        super(NonPolymerChemComp, self).__init__(id, id, id, name=name,
-                                                 formula=formula)
+    def __init__(self, id, code_canonical='X', name=None, formula=None):
+        super(NonPolymerChemComp, self).__init__(id, id, code_canonical,
+                                                 name=name, formula=formula)
 
 
 class WaterChemComp(NonPolymerChemComp):
@@ -864,7 +899,9 @@ class LPeptideAlphabet(Alphabet):
                   ('T', 'THR', 'THREONINE', 'C4 H9 N O3'),
                   ('V', 'VAL', 'VALINE', 'C5 H11 N O2'),
                   ('W', 'TRP', 'TRYPTOPHAN', 'C11 H12 N2 O2'),
-                  ('Y', 'TYR', 'TYROSINE', 'C9 H11 N O3')])
+                  ('Y', 'TYR', 'TYROSINE', 'C9 H11 N O3'),
+                  ('B', 'ASX', 'ASP/ASN AMBIGUOUS', 'C4 H6 N O2 X2'),
+                  ('Z', 'GLX', 'GLU/GLN AMBIGUOUS', 'C5 H8 N O2 X2')])
     _comps['G'] = PeptideChemComp('GLY', 'G', 'G', name='GLYCINE',
                                   formula="C2 H5 N O2")
 
@@ -998,10 +1035,17 @@ class Residue(object):
         return Atom(residue=self, id=atom_id)
 
     def _get_auth_seq_id(self):
-        return self.asym._get_auth_seq_id(self.seq_id)
+        return self.asym._get_auth_seq_id_ins_code(self.seq_id)[0]
     auth_seq_id = property(_get_auth_seq_id,
                            doc="Author-provided seq_id; only makes sense "
                                "for asymmetric units")
+
+    def _get_ins_code(self):
+        return self.asym._get_auth_seq_id_ins_code(self.seq_id)[1]
+    ins_code = property(_get_ins_code,
+                        doc="Insertion code; only makes sense "
+                            "for asymmetric units")
+
     # Allow passing residues where a range is requested
     # (e.g. to ResidueFeature)
     seq_id_range = property(lambda self: (self.seq_id, self.seq_id))
@@ -1113,8 +1157,10 @@ class Entity(object):
     def is_polymeric(self):
         """Return True iff this entity represents a polymer, such as an
            amino acid sequence or DNA/RNA chain (and not a ligand or water)"""
-        return len(self.sequence) != 1 or not isinstance(self.sequence[0],
-                                                         NonPolymerChemComp)
+        return (len(self.sequence) == 0 or
+                len(self.sequence) > 1
+                and any(isinstance(x, (PeptideChemComp, DNAChemComp,
+                                       RNAChemComp)) for x in self.sequence))
 
     def residue(self, seq_id):
         """Get a :class:`Residue` at the given sequence position"""
@@ -1170,6 +1216,22 @@ class AsymUnitRange(object):
     entity = property(lambda self: self.asym.entity)
 
 
+class AsymUnitSegment(object):
+    """An aligned part of an asymmetric unit.
+
+       Usually these objects are created from
+       an :class:`AsymUnit`, e.g. to get a segment covering residues 1 through
+       3 in `asym` use::
+
+           asym = ihm.AsymUnit(entity)
+           seg = asym.segment('--ACG', 1, 3)
+    """
+    def __init__(self, asym, gapped_sequence, seq_id_begin, seq_id_end):
+        self.asym = asym
+        self.gapped_sequence = gapped_sequence
+        self.seq_id_range = (seq_id_begin, seq_id_end)
+
+
 class AsymUnit(object):
     """An asymmetric unit, i.e. a unique instance of an Entity that
        was modeled.
@@ -1179,35 +1241,49 @@ class AsymUnit(object):
        :param str details: Longer text description of this unit.
        :param auth_seq_id_map: Mapping from internal 1-based consecutive
               residue numbering (`seq_id`) to "author-provided" numbering
-              (`auth_seq_id`). This can be either be an int offset, in
-              which case ``auth_seq_id = seq_id + auth_seq_id_map``, or
-              a mapping type (dict, list, tuple) in which case
-              ``auth_seq_id = auth_seq_id_map[seq_id]``. (Note that if a `list`
-              or `tuple` is used, the first element in the list or tuple does
+              (`auth_seq_id` plus an optional `ins_code`). This can be either
+              be an int offset, in which case
+              ``auth_seq_id = seq_id + auth_seq_id_map`` with no insertion
+              codes, or a mapping type (dict, list, tuple) in which case
+              ``auth_seq_id = auth_seq_id_map[seq_id]`` with no insertion
+              codes, or
+              ``auth_seq_id, ins_code = auth_seq_id_map[seq_id]`` - i.e. the
+              output of the mapping is either the author-provided number, or a
+              2-element tuple containing that number and an insertion code.
+              (Note that if a `list` or `tuple` is used for the mapping, the
+              first element in the list or tuple does
               **not** correspond to the first residue and will never be used -
               since `seq_id` can never be zero.) The default if
               not specified, or not in the mapping, is for
-              ``auth_seq_id == seq_id``.
+              ``auth_seq_id == seq_id`` and for no insertion codes to be used.
        :param str id: User-specified ID (usually a string of one or more
               upper-case letters, e.g. A, B, C, AA). If not specified,
               IDs are automatically assigned alphabetically.
+       :param str strand_id: PDB or "author-provided" strand/chain ID.
+              If not specified, it will be the same as the regular ID.
 
        See :attr:`System.asym_units`.
     """
 
-    def __init__(self, entity, details=None, auth_seq_id_map=0, id=None):
+    def __init__(self, entity, details=None, auth_seq_id_map=0, id=None,
+                 strand_id=None):
         self.entity, self.details = entity, details
         self.auth_seq_id_map = auth_seq_id_map
         self.id = id
+        self._strand_id = strand_id
 
-    def _get_auth_seq_id(self, seq_id):
+    def _get_auth_seq_id_ins_code(self, seq_id):
         if isinstance(self.auth_seq_id_map, int):
-            return seq_id + self.auth_seq_id_map
+            return seq_id + self.auth_seq_id_map, None
         else:
             try:
-                return self.auth_seq_id_map[seq_id]
+                ret = self.auth_seq_id_map[seq_id]
+                if isinstance(ret, (int, str)):
+                    return ret, None
+                else:
+                    return ret
             except (KeyError, IndexError):
-                return seq_id
+                return seq_id, None
 
     def __call__(self, seq_id_begin, seq_id_end):
         return AsymUnitRange(self, seq_id_begin, seq_id_end)
@@ -1216,9 +1292,22 @@ class AsymUnit(object):
         """Get a :class:`Residue` at the given sequence position"""
         return Residue(asym=self, seq_id=seq_id)
 
+    def segment(self, gapped_sequence, seq_id_begin, seq_id_end):
+        """Get an object representing the alignment of part of this sequence.
+
+           :param str gapped_sequence: Sequence of the segment, including gaps.
+           :param int seq_id_begin: Start of the segment.
+           :param int seq_id_end: End of the segment.
+        """
+        # todo: cache so we return the same object for same parameters
+        return AsymUnitSegment(self, gapped_sequence, seq_id_begin, seq_id_end)
+
     seq_id_range = property(lambda self: self.entity.seq_id_range,
                             doc="Sequence range")
 
+    strand_id = property(lambda self: self._strand_id or self._id,
+                         doc="PDB or author-provided strand/chain ID")
+
 
 class Assembly(list):
     """A collection of parts of the system that were modeled or probed
